@@ -1,5 +1,5 @@
 /* subprocs.c --- choosing, spawning, and killing screenhacks.
- * xscreensaver, Copyright © 1991-2021 Jamie Zawinski <jwz@jwz.org>
+ * xscreensaver, Copyright © 1991-2025 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -48,12 +48,21 @@
 # define _(S) (S)
 #endif
 
+#if defined(USE_GL) && !defined(HAVE_EGL)
+# include <OpenGL/gl.h>		/* For GLXContext in visual-gl.h */
+# include <GL/glx.h>
+#endif
+
 #include "xscreensaver.h"
 #include "exec.h"
 #include "yarandom.h"
 #include "visual.h"		/* for id_to_visual() */
 #include "atoms.h"
+#include "screenshot.h"
 
+#ifdef USE_GL
+# include "visual-gl.h"
+#endif
 
 enum job_status {
   job_running,	/* the process is still alive */
@@ -65,6 +74,8 @@ enum job_status {
 		   element will be removed. */
 };
 
+#define EXEC_FAILED_EXIT_STATUS -33
+
 struct screenhack_job {
   char *name;
   pid_t pid;
@@ -430,7 +441,7 @@ saver_sigterm_handler (int sig)
   catch_signal (sig, SIG_DFL);
 
   /* The first time a signal fires, inform Xt of that so that it will run
-     xt_signal_handler().  "XtNoticeSignal is the only Intrinsics function
+     xt_sigterm_handler().  "XtNoticeSignal is the only Intrinsics function
      that can safely be called from a signal handler". */
   if (xt_sigterm_id)
     XtNoticeSignal (xt_sigterm_id);
@@ -461,6 +472,16 @@ xt_sigterm_handler (XtPointer data, XtSignalId *id)
         fprintf (stderr, "%s: %s: unblanking\n", blurb(), 
                  signal_name (sigterm_received));
 
+       /* We are in the process of shutting down and are about to exit,
+          so don't accidentally re-launch hacks. */
+      si->terminating_p = True;
+
+      if (si->watchdog_id)
+        {
+          XtRemoveTimeOut (si->watchdog_id);
+          si->watchdog_id = 0;
+        }
+
       /* Kill before unblanking, to stop drawing as soon as possible. */
       for (i = 0; i < si->nscreens; i++)
         {
@@ -485,6 +506,26 @@ xt_sigterm_handler (XtPointer data, XtSignalId *id)
   abort();
 }
 
+#ifdef HAVE_WAYLAND
+/* Called when the Wayland socket drops our connection.
+   We should also die with a SIGPIPE from the XWayland X11 socket,
+   and also a SIGTERM from xscreensaver, so it's a death race.
+ */
+static void
+gfx_wayland_exit (void *closure)
+{
+  saver_info *si = (saver_info *) closure;
+  int i;
+  si->terminating_p = True;
+  for (i = 0; i < si->nscreens; i++)
+    {
+      saver_screen_info *ssi = &si->screens[i];
+      kill_screenhack (ssi);
+    }
+  exit (1);
+}
+#endif /* HAVE_WAYLAND */
+
 
 /* SIGCHLD handling.  Basically the same deal as SIGTERM.
  */
@@ -570,7 +611,11 @@ describe_dead_child (saver_info *si, pid_t kid, int wait_status,
       /* Treat exit code as a signed 8-bit quantity. */
       if (exit_status & 0x80) exit_status |= ~0xFF;
 
-      sprintf (msg, _("crashed with status %d"), exit_status);
+      if (exit_status == EXEC_FAILED_EXIT_STATUS)
+        strcpy (msg, _("is not installed"));
+      else
+        sprintf (msg, _("crashed with status %d"), exit_status);
+
       if (p->verbose_p)
         fprintf (stderr,
                  "%s: %d: child pid %lu (%s) exited abnormally"
@@ -666,6 +711,11 @@ init_sigchld (saver_info *si)
 
   xt_sigchld_id = XtAppAddSignal (si->app, xt_sigchld_handler, si);
   xt_sigterm_id = XtAppAddSignal (si->app, xt_sigterm_handler, si);
+
+# ifdef HAVE_WAYLAND
+  if (si->wayland_dpy)
+    wayland_dpy_atexit (si->wayland_dpy, gfx_wayland_exit, si);
+# endif /* HAVE_WAYLAND */
 }
 
 
@@ -687,10 +737,14 @@ hack_subproc_environment (Screen *screen, Window saver_window)
      them.  In that case, multiple hacks have the same $DISPLAY, screen and
      root window.
    */
+  char *nssw = (char *) malloc (40);
+
+# if !(defined(__APPLE__) && !defined(HAVE_COCOA))  /* not macOS X11 */
+  /* XQuartz's weird $DISPLAY pathnames do not match DisplayString() */
+
   Display *dpy = DisplayOfScreen (screen);
   const char *odpy = DisplayString (dpy);
   char *ndpy = (char *) malloc (strlen(odpy) + 20);
-  char *nssw = (char *) malloc (40);
   char *s, *c;
 
   strcpy (ndpy, "DISPLAY=");
@@ -707,10 +761,11 @@ hack_subproc_environment (Screen *screen, Window saver_window)
   if (s[-1] != '.') *s++ = '.';			/* put on a dot */
   sprintf(s, "%d", screen_number (screen));	/* put on screen number */
 
-  sprintf (nssw, "XSCREENSAVER_WINDOW=0x%lX", (unsigned long) saver_window);
-
   if (putenv (ndpy))
     abort ();
+# endif  /* not macOS X11 */
+
+  sprintf (nssw, "XSCREENSAVER_WINDOW=0x%lX", (unsigned long) saver_window);
   if (putenv (nssw))
     abort ();
 
@@ -767,7 +822,7 @@ fork_and_exec (saver_screen_info *ssi, const char *command)
 
       exec_command (p->shell, command, p->nice_inferior);
       /* If that returned, we were unable to exec the subprocess. */
-      exit (1);  /* exits child fork */
+      exit (EXEC_FAILED_EXIT_STATUS);  /* exits child fork */
       break;
 
     default:	/* parent */
@@ -823,7 +878,7 @@ spawn_screenhack (saver_screen_info *ssi)
   saver_preferences *p = &si->prefs;
   XFlush (si->dpy);
 
-  if (!monitor_powered_on_p (si->dpy))
+  if (!monitor_powered_on_p (si))
     {
       if (si->prefs.verbose_p)
         fprintf (stderr,
@@ -951,6 +1006,11 @@ spawn_screenhack (saver_screen_info *ssi)
 	    goto AGAIN;
 	}
 
+      /* Install screenshot property on window. Must be after
+         select_visual_of_hack() which might replace the window. */
+      if (ssi->screenshot)
+        screenshot_save (si->dpy, ssi->screensaver_window, ssi->screenshot);
+
       if (getuid() == (uid_t) 0 || geteuid() == (uid_t) 0)
         /* Prior to XScreenSaver 6, if running as root, we would change the
            effective uid to the user "nobody" or "daemon" or "noaccess",
@@ -963,6 +1023,27 @@ spawn_screenhack (saver_screen_info *ssi)
           goto DONE;
         }
 
+      if (! si->best_gl_visuals ||
+          ! si->best_gl_visuals[ssi->real_screen_number])
+        {
+          /* Lots of things malfunction in mysterious ways if only *part* of
+             the XScreenSaver application is installed.  That some distros
+             still insist on dividing XScreenSaver into multiple
+             bafflingly-named sub-packages that can be omitted willy nilly
+             causes repeated, predicted, time-wasting and extremely irritating
+             problems for everybody, while solving no problems whatsoever.
+
+             Here's your car, but let's make it trivially easy for everyone to
+             accidentally omit the seat belts and distributor cap, because
+             some weirdo once wanted that in 2002.
+
+             Install all of XScreenSaver or none.
+           */
+          screenhack_obituary (ssi, "",
+            "No GL visuals: the xscreensaver-gl* packages are required.");
+          goto DONE;
+        }
+
       forked = fork_and_exec (ssi, hack->command);
       switch ((int) forked)
 	{
@@ -1060,11 +1141,12 @@ spawn_screenhack (saver_screen_info *ssi)
       ssi->cycle_at = now + how_long / 1000;
 
       if (p->verbose_p)
-        {
-          time_t t = time((time_t *) 0) + how_long/1000;
-          fprintf (stderr, "%s: %d: next cycle in %lu sec at %s\n",
-                   blurb(), ssi->number, how_long/1000, timestring(t));
-        }
+        fprintf (stderr, "%s: %d: next cycle in %d:%02d:%02d at %s\n",
+                 blurb(), ssi->number,
+                 (int)  (how_long / 1000) / (60 * 60),
+                 (int) ((how_long / 1000) % (60 * 60)) / 60,
+                 (int)  (how_long / 1000) % 60,
+                 timestring (ssi->cycle_at));
     }
 }
 
@@ -1083,8 +1165,13 @@ kill_screenhack (saver_screen_info *ssi)
      And some X servers apparently ignore XClearWindow if the monitor is
      powered off, meaning when the monitor powers back on, the stale bits
      that we just tried to erase are still in the frame buffer.
+
+     Do not clear the window when exiting, as that would prevent the
+     unfade from fading the now-dead hack to black, before fading from
+     black to desktop.
    */
-  XClearWindow (si->dpy, ssi->screensaver_window);
+  if (! si->terminating_p)
+    XClearWindow (si->dpy, ssi->screensaver_window);
 }
 
 
@@ -1105,6 +1192,27 @@ any_screenhacks_running_p (saver_info *si)
 }
 
 
+#ifdef USE_GL
+
+Visual *
+get_best_gl_visual (saver_info *si, Screen *screen)
+{
+  saver_preferences *p = &si->prefs;
+  Visual *v = get_gl_visual (screen);
+  if (!v)
+    fprintf (stderr, "%s: %d: no GL visual!\n", blurb(),
+             screen_number (screen));
+  else if (p->verbose_p)
+    fprintf (stderr, "%s: %d: GL visual is 0x%X%s\n",
+             blurb(), screen_number (screen),
+             (unsigned int) XVisualIDFromVisual (v),
+             (v == DefaultVisualOfScreen (screen)
+              ? " (default)" : ""));
+  return v;
+}
+
+#else /* !USE_GL */
+
 /* Fork "xscreensaver-gl-visual" and wait for it to print the IDs of
    the GL visual that should be used on this screen.
  */
@@ -1186,17 +1294,19 @@ get_best_gl_visual (saver_info *si, Screen *screen)
             sprintf (buf, "%s: running %s", blurb(), av[0]);
             perror (buf);
           }
-        exit (1);                               /* exits fork */
+        exit (EXEC_FAILED_EXIT_STATUS);		/* exits fork */
         break;
       }
     default:
       {
         int result = 0;
         int wait_status = 0;
+        int exit_status = EXEC_FAILED_EXIT_STATUS;
         pid_t pid = -1;
         FILE *f;
         unsigned long v = 0;
         char c;
+        int i = 0;
 
         make_job (forked, 0, av[0]);  /* Bookkeeping for SIGCHLD */
 
@@ -1208,8 +1318,13 @@ get_best_gl_visual (saver_info *si, Screen *screen)
         close (out);  /* don't need this one */
 
         *buf = 0;
-        if (! fgets (buf, sizeof(buf)-1, f))
-          *buf = 0;
+        do {
+          errno = 0;
+          if (! fgets (buf, sizeof(buf)-1, f))
+            *buf = 0;
+        } while (errno == EINTR &&	/* fgets might fail due to SIGCHLD. */
+                 i++ < 1000);		/* And just in case. */
+
         fclose (f);
 
         if (! si->prefs.verbose_p)
@@ -1224,6 +1339,16 @@ get_best_gl_visual (saver_info *si, Screen *screen)
           fprintf (stderr, "%s: waitpid(%ld) => %ld\n", blurb(),
                    (long) forked, (long) pid);
 
+        exit_status = WEXITSTATUS (wait_status);
+        /* Treat exit code as a signed 8-bit quantity. */
+        if (exit_status & 0x80) exit_status |= ~0xFF;
+
+        if (exit_status == EXEC_FAILED_EXIT_STATUS)
+          {
+            fprintf (stderr, "%s: %s is not installed\n", blurb(), av[0]);
+            return 0;
+          }
+
         if (1 == sscanf (buf, "0x%lx %c", &v, &c))
           result = (int) v;
 
@@ -1259,3 +1384,5 @@ get_best_gl_visual (saver_info *si, Screen *screen)
 
   abort();
 }
+
+#endif /* !USE_GL */
