@@ -52,6 +52,7 @@
 
 #ifdef USE_GL /* whole file */
 
+#include "doubletime.h"
 
 #define DEF_DELAY            "10000"
 #define DEF_FPS              "False"
@@ -65,7 +66,7 @@
 #define DEF_TEXTURE          "True"
 #define DEF_OFFSET_TEXTURE   "False"
 #define DEF_PAINT_BACKGROUND "True"
-#define DEF_RESOLUTION       "30"
+#define DEF_RESOLUTION       "90"
 #define DEF_BUMPS            "10"
 #define DEF_MOTION_BLUR      "0.0"
 #define DEF_INCREMENTAL      "0"
@@ -311,26 +312,6 @@ typedef struct {
 
 static mirrorblobstruct *Mirrorblob = NULL;
 
-/******************************************************************************
- *
- * Returns the current time in seconds as a double.  Shamelessly borrowed from
- * glslideshow.
- *
- */
-static double
-double_time (void)
-{
-  struct timeval now;
-# ifdef GETTIMEOFDAY_TWO_ARGS
-  struct timezone tzp;
-  gettimeofday(&now, &tzp);
-# else
-  gettimeofday(&now);
-# endif
-
-  return (now.tv_sec + ((double) now.tv_usec * 0.000001));
-}
-
 /******************************************************************************
  *
  * Change to the projection matrix and set our viewing volume.
