@@ -13,6 +13,14 @@
 #define POW2_H
 
 #include <stdlib.h>
+#include <limits.h>
+
+/* Like i_log2, but no check for 0. */
+#if defined __GNUC__ && __GNUC__ >= 4 || defined __clang__
+# define i_log2_fast(x) (sizeof(long) * CHAR_BIT - __builtin_clzl(x) - 1)
+#else
+# define i_log2_fast(x) i_log2(x)
+#endif
 
 extern int i_log2(size_t x);
 extern size_t to_pow2 (size_t x); /* return the next larger power of 2. */
